/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.3 $
 * $Date: 2004/03/30 15:47:06 $
 *
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>

#include "global.h"
#include "Connection.h"

Connection::Connection(Server* server)
{
  debug("Connection::Connection(Server* server)\n");
  this->server = server;
  this->status = CONNECTION_STATUS_DISCONNECTED;
  this->bufferLength = 0;
  this->bufferPos = 0;
}

Connection::~Connection()
{
  debug("Connection::~Connection()\n");
  if (this->status == CONNECTION_STATUS_CONNECTED)
    this->disconnect();
}

int Connection::connect()
{
  debug("Connection::connect()\n");
  if (this->status == CONNECTION_STATUS_CONNECTED)
    return 0;

  int res = this->doConnect();

  if (res >= 0)
    this->status = CONNECTION_STATUS_CONNECTED;
  else
    Connection::doDisconnect();

  return res;
}

int Connection::disconnect()
{
  debug("Connection::disconnect()\n");
  if (this->status == CONNECTION_STATUS_DISCONNECTED)
    return 0;

  int res = this->doDisconnect();

  this->status = CONNECTION_STATUS_DISCONNECTED;

  return res;  
}

int Connection::write(char* line)
{
  debug("Connection::write(char* line)\n");
  if (this->status == CONNECTION_STATUS_DISCONNECTED) {
    int res = this->connect();
    if (res < 0)
      return res;
  }

  int res = this->doWrite(line);
  
  if (res == EOF)
    Connection::doDisconnect();
  
  return res;
}


char* Connection::readLine()
{
  //  debug("Connection::readLine()\n");
  if (this->status == CONNECTION_STATUS_DISCONNECTED) {
    int res = this->connect();
    if (res < 0)
      return NULL;
  }

  char* res = this->doReadLine();
  
  if (res == NULL)
    Connection::doDisconnect();
  
  return res;
}

int Connection::doConnect()
{
  debug("Connection::doConnect()\n");
  //resolve hostname

  this->bufferLength = 0;
  this->bufferPos = 0;

  struct hostent *hinfo;

  hinfo = ::gethostbyname(this->server->getHost());

  if (!hinfo)
    {
      error("Could not resolve hostname %s", this->server->getHost());
      return -1;
    }

  struct sockaddr_in sa_serv;
  memset(&sa_serv, '\0', sizeof(sa_serv));
  sa_serv.sin_family = AF_INET;
  memcpy(&sa_serv.sin_addr.s_addr, hinfo->h_addr_list[0], sizeof(sa_serv.sin_addr.s_addr));
  sa_serv.sin_port = htons(this->server->getPort());

  this->socket = ::socket(PF_INET,SOCK_STREAM,0);

  if (this->socket < 0)
    {
      error("Socket creation failed for %s!", this->server->getHost());
      return -1;
    }

  int res = ::connect(this->socket,(struct sockaddr *) &sa_serv, sizeof(sa_serv));

  if (res < 0)
    {
      error("Connection to %s failed!", this->server->getHost());
      return -1;
    }

  this->fbuffer = fdopen(this->socket,"r+");

  if (!this->fbuffer)
    {
      error("fdopen failed for %s", this->server->getHost());
      return -1;
    }
  return 0;
}

int Connection::doDisconnect()
{
  debug("Connection::doDisconnect()\n");
  if (this->fbuffer)
    fclose(this->fbuffer);
  if (this->socket > -1)
    close(this->socket);
  
  this->socket = -1;
  this->fbuffer = NULL;

  this->status = CONNECTION_STATUS_DISCONNECTED;

  return 0;
}

int Connection::doWrite(char* text)
{
  debug("Connection::doWrite(char* text)\n");
  return ::fputs(text, this->fbuffer);
}

char* Connection::doReadLine()
{
  return ::fgets(this->readBuffer, 10240, this->fbuffer);
}

