/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.6 $
 * $Date: 2004/03/30 12:21:03 $
 *
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <unistd.h>

#include "Coordinator.h"
#include "Job.h"
#include "DownloadJob.h"
#include "FileDownloader.h"
#include "MultiServerPool.h"
#include "Frontend.h"


Coordinator::Coordinator(Queue* queue, Options* options)
{
  this->queue = queue;
  this->options = options;
}

void Coordinator::start()
{

  std::vector<FileDownloader*> children;

  children.clear();

  while (1)
    {
      Job* job = queue->getNextJob();
      
      if (!job)
        break;

      Frontend::getInstance()->setQueueSize(queue->getWaitingSize());
      
      if (job->getType() == JOBTYPE_DOWNLOAD)
        {
          FileDownloader* downer = 
            new FileDownloader((DownloadJob*)job, 
                               this->options->getOption("tempdir","."), 
                               this->options->getOption("destinationdir","."));
          children.push_back(downer);
          downer->start();
        }

//       usleep(1000*1000);

//       //wait until every level has a free connection
//       int level = 0;
//       while (1)
//         {
//           NNTPConnection* conn = MultiServerPool::getInstance()->getConnection(level);
//           if (conn)
//             MultiServerPool::getInstance()->freeConnection(level, conn);
//           else
//             break;
//           level++;
//         }

      // cleanup finished jobs
      for (unsigned int i = 0; i < children.size(); i++)
        {
          if ((children[i]) && (!children[i]->isRunning()))
            {
              delete children[i];
              children[i] = NULL;
            }
        }
    }

  // no more jobs!
  // wait for running ones to finish

  while (1)
    {
      bool stop = true;
      
      for (unsigned int i = 0; i < children.size(); i++)
        {
          if (children[i])
            {
              if (children[i]->isRunning())
                {
                  stop = false;
                }
              else
                {
                  delete children[i];
                  children[i] = NULL;
                }
            }
        }

      if (stop)
        break;
      
      usleep(1000*1000);
    }
}
