/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.3 $
 * $Date: 2004/02/25 12:46:10 $
 *
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define PROTOTYPES

#include <stdlib.h>
#include <stdio.h>
#include <uudeview.h>
#include <pthread.h>
#include <string.h>

#include "Decoder.h"
#include "global.h"

Decoder::Decoder()
{
  debug("Decoder::Decoder()");
  pthread_mutex_init(&mutex_decoder, NULL);
}

char* Decoder::decode(char* filename, char* destdir)
{

  char* res = NULL;

  pthread_mutex_lock(&mutex_decoder);

  UUInitialize();

  UUSetOption(UUOPT_DESPERATE,1,NULL);
  //  UUSetOption(UUOPT_DUMBNESS,1,NULL);
  UUSetOption(UUOPT_SAVEPATH,1,destdir);

  UULoadFile(filename, NULL, 0);

  // choose right attachment

  uulist* attachment = NULL;

  for (int i = 0; ; i++)
    {
      uulist* att_tmp = UUGetFileListItem(i);
      
      if (!att_tmp)
        break;

      if ((att_tmp) && (att_tmp->haveparts))
        {
          if (!attachment)
            {
              attachment = att_tmp;
            }
          else
            {
              //f**k, multiple attachments!? Can't handle this.
              attachment = NULL;
              break;
            }
        }
    }

  if (attachment)
    {
      // okay, we got only one attachment, perfect!
      if ((attachment->haveparts) && (attachment->haveparts[0])) //  && (!attachment->haveparts[1]))  FUCK UULIB
        {
          // only one part, even better! (har har)
          char tmp[1024];

          if (attachment->filename)
            {
              if (attachment->uudet == UU_ENCODED)
                {
                  // yep, I love uulib... file-part-detection is very buggy, so
                  // do not trust it!
                  // we know the filename, so part must be 1 !! :-)
                  snprintf(tmp, 1024, "%s.1", attachment->filename);
                }
              else
                snprintf(tmp,1024,"%s.%d", attachment->filename, attachment->haveparts[0]);
            }
          else
            {
              // mmkay, filename unknown... do something clever!
              // how about renaming it to some known pattern and letting
              // FileDownloader handle it?

              snprintf(tmp, 1024, "nzbgettemp-unknown-XXXXXX");
              mktemp(tmp);
            }
          
          UURenameFile(attachment, tmp);
          
          int r = UUDecodeFile(attachment, NULL);
          
          if (r == UURET_OK)
            {
              // we did it!
              res = strdup(tmp);
            }
        }
      else
        {
          error("[ERROR] Wrong number of parts!\n");
        }
    }
  else
    error("[ERROR] Wrong number of attachments!\n");

  UUCleanUp();

  pthread_mutex_unlock(&mutex_decoder);

  return res;
  
}

Decoder* Decoder::getInstance()
{
  static Decoder singleton;
  return &singleton;
}
