/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.4 $
 * $Date: 2004/03/30 12:21:03 $
 *
 */

#ifndef FRONTEND_H
#define FRONTEND_H

#include <vector>
#include <pthread.h>
#include <time.h>

#include "Thread.h"


typedef struct _runningDL
{
    char* messageid;
    char* server;
    int bytes;
    int lines;
    int maxlines;
    time_t starttime;
} runningDL;


class Frontend : public Thread
{
 public:

    Frontend();

    virtual ~Frontend();

    static Frontend* getInstance();

    void started(char* messageid, char* server);
    
    void finished(char* messageid, char* server);

    void progress(char* messageid, char* server, int bytes, int lines, int maxlines);

    void frontendError(char* msg, ...);

    void frontendWarn(char* msg, ...);

    void frontendDebug(char* msg, ...);

    void frontendInfo(char* msg, ...);
    
    void setQueueSize(long long s) { this->queueSize = s; }


 protected:

    virtual void run();

    void printMessages();

    void printStatus();

    std::vector< char* > messages;

    std::vector< runningDL* > running;

    pthread_mutex_t mutex_running;

    pthread_mutex_t mutex_messages;

    long long queueSize;
};

#endif
