/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.6 $
 * $Date: 2004/03/30 15:47:07 $
 *
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "NZBQueue.h"
#include "global.h"
#include "DownloadJob.h"



NZBQueue::NZBQueue(xmlDocPtr nzbdoc)
{
  this->jobs.clear();
  this->parseNZB(nzbdoc);
}

NZBQueue::~NZBQueue()
{
  for (unsigned int i=0; i < this->jobs.size(); i++) {
    delete this->jobs[i];
    this->jobs[i] = NULL;
  }
  this->jobs.clear();
}


Job* NZBQueue::getNextJob()
{
  Job* j = NULL;
  
  if (this->jobs.size()>0)
    {
      j = this->jobs[0];
      this->jobs.erase(this->jobs.begin());
    }

  return j;
}

void NZBQueue::parseSegment(xmlNodePtr node, std::vector<char*>* idlist, long long* size)
{
  if (!node)
    return;
  
  if ((node->name) && (!strcmp((char*)node->name, "segment")))
    {
      //segment-tag found
      if ((node->children) && (node->children->content))
        {
          char tmp[2048];
          snprintf(tmp, 2048, "<%s>", (char*)node->children->content);
          idlist->push_back(strdup(tmp));
        }

      xmlAttr* attr = node->properties;

      while (attr)
        {
          if ((attr->name) && (!strcmp((char*)attr->name, "bytes")) && (attr->children))
            {
              long long lsize = atol((char*) attr->children->content);
              if (lsize > 0)
                *size += lsize;
              break;
            }
          attr = attr->next;
        }

      this->parseSegment(node->next, idlist, size);
    }
  else
    {
      this->parseSegment(node->next, idlist, size);
      this->parseSegment(node->children, idlist, size);
    }
}

void NZBQueue::parseGroup(xmlNodePtr node, std::vector<char*>* grouplist)
{
  if (!node)
    return;
  
  if ((node->name) && (!strcmp((char*)node->name, "group")))
    {
      //group-tag found
      if ((node->children) && (node->children->content))
        {
          char tmp[2048];
          snprintf(tmp, 2048, "%s", (char*)node->children->content);
          grouplist->push_back(strdup(tmp));
        }
      this->parseGroup(node->next, grouplist);
    }
  else
    {
      this->parseGroup(node->next, grouplist);
      this->parseGroup(node->children, grouplist);
    }
}

void NZBQueue::parseFile(xmlNodePtr node)
{
  if (!node)
    return;

  if ((node->name) && (!strcmp((char*)node->name, "file")))
    {
      //file-tag found
      DownloadJob* job = new DownloadJob();

      xmlAttr* attr = node->properties;

      while (attr)
        {
          if ((attr->name) && (!strcmp((char*)attr->name, "subject")) && (attr->children))
            {
              job->setSubject(strdup((char*)attr->children->content));
              break;
            }
          attr = attr->next;
        }

      long long size = 0;

      this->parseSegment(node->children, job->getMessageIDs(), &size);
      this->parseGroup(node->children, job->getGroups());
      job->setSize(size);
      this->jobs.push_back(job);
      this->parseFile(node->next);
    }
  else
    {
      this->parseFile(node->children);
      this->parseFile(node->next);
    }

  debug("%s",(char*)node);
}

void NZBQueue::parseNZB(xmlDocPtr nzb)
{
  // walk through whole doc and search for segments-tags
  this->parseFile(nzb->children); 
}

long long NZBQueue::getWaitingSize()
{
  long long result = 0;
  
  for (unsigned int i = 0; i < this->jobs.size(); i++)
    {
      if ((this->jobs[i]) && (this->jobs[i]->getType() == JOBTYPE_DOWNLOAD))
        result += ((DownloadJob*)this->jobs[i])->getSize();
      
    }
  return result;
}
