/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.1 $
 * $Date: 2004/02/24 11:05:48 $
 *
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <libxml/xpath.h>

#include "Options.h"
#include "global.h"

#include "SingleServerPool.h"
#include "MultiServerPool.h"
#include "NewsServer.h"



Options::Options(char* optfile)
{
  this->optdoc = xmlParseFile(optfile);

  if (!this->optdoc)
    {
      error("FATAL ERROR: Configuration file %s not found or broken! Aborting.\n", optfile);
      exit(-1);
    }

  this->initServers();

  //find 
  
}

char* Options::getOption(char* optname, char* defaultValue)
{

  char* res = defaultValue;

  xmlXPathContextPtr context;
  xmlXPathObjectPtr result;

  context = xmlXPathNewContext(this->optdoc);

  char tmp[1024];
  snprintf(tmp, 1024, "//option[@name='%s']", optname);
  
  result = xmlXPathEvalExpression((xmlChar *) tmp, context);

  if ((result) && (result->nodesetval) && (result->nodesetval->nodeTab) &&
      (result->nodesetval->nodeTab[0]) && (result->nodesetval->nodeTab[0]->children) &&
      (result->nodesetval->nodeTab[0]->children->content))

    res = (char*)result->nodesetval->nodeTab[0]->children->content;

  if (result)
    xmlXPathFreeObject(result);
  xmlXPathFreeContext(context);

  return res;
}


void Options::initServers()
{

  for (int level=0; level < 100; level++)
    {

      xmlXPathContextPtr context;
      xmlXPathObjectPtr result;

      context = xmlXPathNewContext(this->optdoc);

      char tmp[1024];
      snprintf(tmp, 1024, "//server[@level='%d']", level);
      
      result = xmlXPathEvalExpression((xmlChar *) tmp, context);
      
      if ((result) && (result->nodesetval) && (result->nodesetval->nodeTab) &&
          (result->nodesetval->nodeTab[0]))
        {
          xmlNodePtr servernode = result->nodesetval->nodeTab[0];

          char* host = NULL;
          int port = 119;
          char* user = NULL;
          char* pass = NULL;
          int conns = 1;

          xmlNodePtr child = servernode->children;

          while (child)
            {
              if (child->children)
                {
                  if (!strcmp((char*)child->name, "host"))
                    host = (char*)child->children->content;
                  if (!strcmp((char*)child->name, "port"))
                    port = atoi((char*)child->children->content);
                  if (!strcmp((char*)child->name, "user"))
                    user = (char*)child->children->content;
                  if (!strcmp((char*)child->name, "pass"))
                    pass = (char*)child->children->content;
                  if (!strcmp((char*)child->name, "connections"))
                    conns = atoi((char*)child->children->content);
                }
              child = child->next;
            }

          if (host)
            {
              NewsServer* serv = new NewsServer(host, port, user, pass);
              SingleServerPool* pool = new SingleServerPool(serv, conns);
              MultiServerPool::getInstance()->addServerPool(pool);
            }
        }
      else
        level = 1000;
      
      if (result)
        xmlXPathFreeObject(result);
      xmlXPathFreeContext(context);
    }
}
