/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.3 $
 * $Date: 2004/03/30 12:21:03 $
 *
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <unistd.h>

#include "SingleServerPool.h"
#include "global.h"

SingleServerPool::SingleServerPool()
{
  debug("SingleServerPool::SingleServerPool()\n");
  this->server = NULL;
  this->max = 0;
  this->id = 0;
  this->connections.clear();
  this->free.clear();
  this->queue.clear();
  

  pthread_mutex_init(&mutex_queue, NULL);
  pthread_mutex_init(&mutex_free, NULL);
}

int SingleServerPool::getID()
{
  debug("SingleServerPool::getID() : Assigning ID %d\n",id+1);
  return ++id;
}


SingleServerPool::SingleServerPool(NewsServer* s, int max)
{
  debug("SingleServerPool::SingleServerPool(NewsServer*, int)\n");
  this->server = s;
  this->max = max;
  this->id = 0;
  this->connections.clear();
  this->free.clear();
  this->queue.clear();

  pthread_mutex_init(&mutex_queue, NULL);
  pthread_mutex_init(&mutex_free, NULL);

  for (int i = 0; i < this->max; i++)
    {
      this->connections.push_back(new NNTPConnection(this->server));
      this->free.push_back(i);
    }
};


NNTPConnection* SingleServerPool::getConnection()
{

  debug("SingleServerPool::getConnection() : Starting connection-request...\n");

  int connection_nr = -1;

  // get waiting-ID
  int my_id = this->getID();

  // lock queue
  pthread_mutex_lock(&mutex_queue);

  // append to queue
  this->queue.push_back(my_id);

  // unlock queue
  pthread_mutex_unlock(&mutex_queue);

  // wait until any connection is free
  while (1)
    {
      // lock free
      pthread_mutex_lock(&mutex_free);

      // something free?
      if (this->free.size()>0)
        {
          // got one!
          // may I take it?
          pthread_mutex_lock(&mutex_queue);
          if ((this->queue.size()>0) && (this->queue[0] == my_id))
            {
              // oh yeah, it's mine!
              this->queue.erase(this->queue.begin());
              connection_nr = this->free[this->free.size()-1];
              this->free.pop_back();
            }
          pthread_mutex_unlock(&mutex_queue);
        }

      // unlock free
      pthread_mutex_unlock(&mutex_free);

      if (connection_nr!=-1)
        break;
      
      usleep(100*1000); // wait 100 msec
    }

  debug("SingleServerPool::getConnection() : Assigned connection %d to wait-ID %d\n",
        connection_nr, my_id);

  return connections[connection_nr];
}

void SingleServerPool::freeConnection(NNTPConnection* con)
{
  for (int i = 0; i < this->max; i++)
    {
      if (this->connections[i] == con) {
        debug("SingleServerPool::freeConnection(NNTPConnection*) : Connection %d freed\n",i);
        pthread_mutex_lock(&mutex_free);
        this->free.push_back(i);
        pthread_mutex_unlock(&mutex_free);
      }
    }
}
