/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.2 $
 * $Date: 2004/02/24 22:30:40 $
 *
 */

#ifndef THREAD_H
#define THREAD_H

#include <pthread.h>

/**
 * Generic class for all classes running a second POSIX thread.
 * Just take care that the run() method is overloaded and the 
 * start() routine is called.
 */
class Thread
{
public:

  /**
   * Default constructor.
   * Sets some parameters for the 2nd thread.
   */
  Thread();

  /**
   * Destructor.
   */
  virtual ~ Thread();

  /**
   * Starts the 2nd thread.
   */
  virtual void start();

  /**
   * Stops the 2nd thread.
   */
  virtual void stop();

  /**
   * Get/Set the flag that indicates if the 2nd thread is running.
   */
  bool isRunning() { return this->runFlag; }

  void setRunning(bool v) { this->runFlag = v; }

  /**
   * Get/Set the record representing the 2nd thread.
   */
  pthread_t getThread() { return this->thread; }

  void setThread(pthread_t v) { this->thread = v; }

protected:

  /**
   * Record representing the 2nd thread.
   */
  pthread_t thread;

  /**
   * Attributes for the 2nd thread.
   */
  pthread_attr_t attr;

  /**
   * Flag that indicates if the 2nd thread is running.
   */
  bool runFlag;

  /**
   * Activities of the 2nd thread.
   * This should be overloaded in derived classes.
   */
  virtual void run()
  {
  }

  /**
   * Static function calling the run() method of the specified 
   * Thread object.
   * @param obj Thread object whose run() method should be executed as 
   *        2nd thread.
   * @return NULL
   */
  static void *thread_handler(void *obj);

private:
};

#endif
