/*
 *  This file if part of nzbget
 *
 *  Copyright (C) 2004  Sven Henkel <sidddy@users.sourceforge.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Revision: 1.2 $
 * $Date: 2004/02/25 23:11:53 $
 *
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <unistd.h>
#include <pwd.h>
#include <sys/types.h>

#include <libxml/parser.h>

#include "MultiServerPool.h"
#include "global.h"
#include "NZBQueue.h"
#include "Options.h"
#include "Coordinator.h"
#include "Frontend.h"



int main(int argc, char *argv[])
{

  printf("nzbget v"VERSION"\n");

  if (argc != 2)
    {
      printf("Usage: %s <nzb-file>\n\n", argv[0]);
      return 0;
    }

  char* home = getenv("HOME");
  if (!home)
    {
      struct passwd *pw = getpwuid(getuid());
      if (pw)
        home = pw->pw_dir;
    }

  if (!home)
    {
      printf("FATAL ERROR: Unable to determine home-directory. Aborting!\n");
      exit(-1);
    }

  char conffn[1024];

  if (home[strlen(home)-1] == '/')
    snprintf(conffn, 1024, "%s.nzbget/nzbget.cfg", home);
  else
    snprintf(conffn, 1024, "%s/.nzbget/nzbget.cfg", home);
    
  Options* options = new Options(conffn);

  xmlDocPtr doc = xmlParseFile(argv[1]);

  xmlCleanupParser();

  if (!doc)
    {
      printf("Parsing NZB-document failed!!\n\n");
      return -1;
    }

  NZBQueue* queue = new NZBQueue(doc);

  xmlFreeDoc(doc);

  

  Coordinator* coor = new Coordinator(queue, options);

  Frontend::getInstance()->start();

  coor->start();

  return 0;

}
